#include <cabling/cabling.h>
#include <soapCablingImplPortBindingProxy.h>
#include <soapH.h>
#include <string>
#include <sstream>
#include <base/CDBException.h>
#include <base/CDBPermanentException.h>
#include <base/CDBResponseHandler.h>

namespace MAUS {
    namespace CDB {
/**
 * Obtains detector Cabling parameters as a string. An exception is thrown when result is null
 * or contains an error or warning (CDB specific) XML message.
 * @param device  i.e. "TOF0"
 * @param runNo   i.e. 8423
 * @param result  a c++ string containing the result
 */
        void Cabling::getDetectorCablingForRun(const char* device, int runNo, std::string & result) {

            //char *ret = NULL;
            //char *retcopy = NULL;

            CDBResponseHandler handler;

            ns1__getDetectorCablingForRun request;
            ns1__getDetectorCablingForRunResponse response;

            // arg0/1 are pointers to std::string

            // std::string s = std::to_string(runNo);  C++11
            // std::cout<< s << std::endl;

            std::stringstream ss;
            ss << runNo;
            std::string run = ss.str();
            request.arg1 = &run;

            std::string sdevice = std::string(device);
            request.arg0 = &sdevice;

            const char *soap_action = NULL;

            /* invoke the web service operation */

            if (proxy.getDetectorCablingForRun(endpoint, soap_action, &request, response) == SOAP_OK) {
                std::cout << "SOAP_OK! " << std::endl;
                // assign the server response to the result string:
                // __ptr points to a non-terminated chunk of memory. The assignment solves this problem.
                if (NULL != response.return_->__ptr) {
                    result.assign((char*) response.return_->__ptr, response.return_->__size);
                } else {
                    std::string nullResp("getDetectorCablingForRun: CDB server returned no data (response buffer is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                // now check if response is not simply an error or warning XML document
                handler.analyze(result.c_str());

            } else {
                proxy.soap_stream_fault(std::cerr);
                std::string pre("SOAP fault: ");
                std::string fault((char*) proxy.soap_fault_string());
                std::string detail((char*) proxy.soap_fault_detail());
                throw CDBException(pre.append(fault).append("\nDetail:").append(detail));
            }

            return;
        }
/**
 * Obtains detector Cabling parameters as a string. An exception is thrown when result is null
 * or contains an error or warning (CDB specific) XML message.
 * @param device  i.e. "TOF0"
 * @param timestamp  i.e. "2013-08-01 00:00:00"
 * @param result  a c++ string containing the result
 */
        void Cabling::getDetectorCablingForDate(const char* device, char* timestamp, std::string& result) {

            ns1__getDetectorCablingForDate request;
            ns1__getDetectorCablingForDateResponse response;

            CDBResponseHandler handler;

            std::string sdevice = std::string(device);
            request.arg0 = &sdevice;

            std::string _timestamp = std::string(timestamp);
            request.arg0 = &_timestamp;

            const char *soap_action = NULL;
           
            if (proxy.getDetectorCablingForDate(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_->__ptr) {
                    result.assign((char*) response.return_->__ptr, response.return_->__size);
                } else {
                    std::string nullResp("getDetectorCablingForDate: CDB server returned no data (response buffer is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                // now check if response is not simply an error or warning XML document
                handler.analyze(result.c_str());
            } else {
                wrapCDBException(); // build an exception string and throw
            }

            return;
        }
/**
 * Obtains current valid detector Cabling parameters as a string. An exception is thrown when result is null
 * or contains an error or warning (CDB specific) XML message.
 * @param device i.e. "TOF0"
 * @param result  a c++ string containing the result
 */
        void Cabling::getCurrentDetectorCabling(const char* device, std::string& result) {

            ns1__getCurrentDetectorCabling request;
            ns1__getCurrentDetectorCablingResponse response;

            CDBResponseHandler handler;

            std::string sdevice = std::string(device);
            request.arg0 = &sdevice;

            const char *soap_action = NULL;

            if (proxy.getCurrentDetectorCabling(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_->__ptr) {
                    result.assign((char*) response.return_->__ptr, response.return_->__size);
                } else {
                    std::string nullResp("getCurrentDetectorCabling: CDB server returned no data (response buffer is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                // now check if response is not simply an error or warning XML document
                handler.analyze(result.c_str());
            } else {
                wrapCDBException(); // build an exception string and throw
            }


            return;
        }
/**
 * Obtains detector Cabling parameters as a string for a given id. An exception is thrown when result is null
 * or contains an error or warning (CDB specific) XML message.
 * @param id  Cabling id
 * @param result a c++ string containing the result
 */
        void Cabling::getDetectorCablingForId(int id, std::string& result){

            ns1__getDetectorCablingForId request;
            ns1__getDetectorCablingForIdResponse response;

            CDBResponseHandler handler;

            const char *soap_action = NULL;
            
            std::string s_id = std::to_string(id); // C++ 11
            request.arg0 = &s_id;

            if (proxy.getDetectorCablingForId(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_->__ptr) {
                    result.assign((char*) response.return_->__ptr, response.return_->__size);
                } else {
                    std::string nullResp("getDetectorCablingForId: CDB server returned no data (response buffer is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                // now check if response is not simply an error or warning XML document
                handler.analyze(result.c_str());
            } else {
                wrapCDBException(); // build an exception string and throw
            }

            return;
        }

        void Cabling::getStatus(std::string& result) {
            
            ns1__getStatus request ;
            ns1__getStatusResponse response ;
            CDBResponseHandler handler;
            const char *soap_action = NULL;
            
            if (proxy.getStatus(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_) {
                    handler.analyze(response.return_->c_str()); // look for ok, warning, error. Last 2 throw an exception
                    result.assign(handler.getMessage());
                } else {
                    std::string nullResp("getStatus: CDB server returned no data (response std::string* is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                
            } else {
                wrapCDBException(); // build an exception string and throw
            }

            return;
            
        }
        
        void Cabling::wrapCDBException() {

            proxy.soap_stream_fault(std::cerr);
            std::string pre("SOAP fault: ");
            std::string fault((char*) proxy.soap_fault_string());
            std::string detail((char*) proxy.soap_fault_detail());
            throw CDBException(pre.append(fault).append("\nDetail:").append(detail));
        }

        Cabling::~Cabling() {
            /* Delete data and free memory */
            proxy.destroy();
            std::cout << "Cabling proxy destroyed" << std::endl;
        }

    }
}